// <<--------------------------------------------->>
// << SLADE (SlayeR's 'LeetAss Doom Editor)       >>
// << By Simon Judd, 2004                         >>
// << ------------------------------------------- >>
// << <gui.cpp>                                   >>
// << GUI drawing/handling functions              >>
// <<--------------------------------------------->>

// Disable conversion warnings
#pragma warning (disable: 4244)

// Includes ------------------------------------- >>
#include "main.h"
#include "draw.h"
#include "map.h"
#include "menu.h"
//#include "args.h"

// Variables ------------------------------------ >>
rgba_t col_boxhead(0, 180, 250, 255);
rgba_t col_boxbody(255, 255, 255, 255);
rgba_t col_boxhead_text(0, 220, 255, 255);
rgba_t col_boxbody_text(0, 0, 0, 255);
rgba_t col_menu(0, 180, 250, 255);
rgba_t col_menuhead_text(255, 255, 255, 200);
rgba_t col_menubuttons(255, 255, 255, 255);
rgba_t col_popupmenu(180, 180, 180, 255);
rgba_t col_menutext(0, 0, 0, 255);
rgba_t col_menuselect(0, 180, 250, 150);

float x_scale;
float y_scale;

//rgba_t col_white(255, 255, 255, 255);
//rgba_t col_black(0, 0, 0, 255);

CVAR(Bool, gui_scale, false, CVAR_SECRET)

// External Variables --------------------------- >>
extern Map map;
extern int gridsize;
extern point2_t mouse;
extern Uint8* keys;
extern BYTE sel_item;
extern string menu_input;
extern bool sel_items[16];
extern string sector_types[256];
extern rect_t menu_area;
extern BYTE open_menu, line_side;
EXTERN_CVAR(Int, vid_width)
EXTERN_CVAR(Int, vid_height)
EXTERN_CVAR(Int, vid_fullscreen)

void update_gui_scale()
{
	if (gui_scale)
	{
		x_scale = (float)vid_width / 800.0f;
		y_scale = (float)vid_height / 600.0f;
	}
	else
		x_scale = y_scale = 1.0f;
}

// gui_drawsplash: Draws the splash screen with a message
// --------------------------------------------------- >>
void gui_drawsplash(const char* message)
{
	int x = (vid_width / 2) - 160 * x_scale;
	int y = (vid_height / 2) - 100 * y_scale;

	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	draw_texture("_title", x, y, 320 * x_scale, 200 * y_scale, COL_WHITE);
	draw_text(x + (16 * x_scale), y + (184 * y_scale), COL_WHITE, 0, message);
	SDL_GL_SwapBuffers();
}

// gui_drawbox: Draws an empty info box
// --------------------------------- >>
void gui_drawbox(int x, int y, int width, char* title, ...)
{
	char string[256];
	va_list ap;

	va_start(ap, title);
	vsprintf(string, title, ap);
	va_end(ap);

	// Draw box head
	draw_texture_rect("_boxhead", x, y, 16 * x_scale, 16 * y_scale, rect_t(0, 0, 16, 16), col_boxhead);
	draw_texture_rect("_boxhead", x + 16 * x_scale, y, width - 32 * x_scale, 16 * y_scale, rect_t(16, 0, 48, 16), col_boxhead);
	draw_texture_rect("_boxhead", (x + width) - 16 * x_scale, y, 16 * x_scale, 16 * y_scale, rect_t(48, 0, 64, 16), col_boxhead);
	draw_text(x + (width / 2) + 1, y + 5, COL_BLACK, 1, string);
	draw_text(x + (width / 2), y + 4, col_boxhead_text, 1, string);

	// Draw box
	draw_texture_rect("_box", x, y + 16 * y_scale, 16 * x_scale, 128 * y_scale, rect_t(0, 0, 16, 128), col_boxbody);
	draw_texture_rect("_box", x + 16 * x_scale, y + 16 * y_scale, width - 32 * x_scale, 128 * y_scale, rect_t(16, 0, 48, 128), col_boxbody);
	draw_texture_rect("_box", (x + width) - 16 * x_scale, y + 16 * y_scale, 16 * x_scale, 128 * y_scale, rect_t(48, 0, 64, 128), col_boxbody);
}

// gui_drawpopup: Draws an empty popup menu
// ------------------------------------- >>
void gui_drawpopup(int x, int y, int width, int height, int border)
{
	int rwidth = width - (border * 2);
	int rheight = height - (border * 2);
	int bs = border;

	draw_texture_rect("_menu", x, y, bs, bs, rect_t(0, 0, 16, 16), col_popupmenu);
	draw_texture_rect("_menu", x + bs, y, rwidth, bs, rect_t(16, 0, 48, 16), col_popupmenu);
	draw_texture_rect("_menu", x + width - bs, y, bs, bs, rect_t(48, 0, 64, 16), col_popupmenu);

	draw_texture_rect("_menu", x, y + bs, bs, rheight, rect_t(0, 16, 16, 112), col_popupmenu);
	draw_texture_rect("_menu", x + bs, y + bs, rwidth, rheight, rect_t(16, 16, 48, 112), col_popupmenu);
	draw_texture_rect("_menu", x + width - bs, y + bs, bs, rheight, rect_t(48, 16, 64, 112), col_popupmenu);

	draw_texture_rect("_menu", x, y + height - bs, bs, bs, rect_t(0, 112, 16, 128), col_popupmenu);
	draw_texture_rect("_menu", x + bs, y + height - bs, rwidth, bs, rect_t(16, 112, 48, 128), col_popupmenu);
	draw_texture_rect("_menu", x + width - bs, y + height - bs, bs, bs, rect_t(48, 112, 64, 128), col_popupmenu);
}

// gui_drawpopup: Draws an empty popup menu with a title
// -------------------------------------------------- >>
void gui_drawpopup(int x, int y, int width, int height, char* title, ...)
{
	char string[256];
	va_list ap;

	va_start(ap, title);
	vsprintf(string, title, ap);
	va_end(ap);

	// Draw title
	draw_texture_rect("_boxhead", x, y, 16, 16, rect_t(0, 0, 16, 16), col_boxhead);
	draw_texture_rect("_boxhead", x + 16, y, width - 32, 16, rect_t(16, 0, 48, 16), col_boxhead);
	draw_texture_rect("_boxhead", (x + width) - 16, y, 16, 16, rect_t(48, 0, 64, 16), col_boxhead);
	draw_text(x + (width / 2) + 1, y + 5, COL_BLACK, 1, string);
	draw_text(x + (width / 2), y + 4, col_menuhead_text, 1, string);

	// Draw menu
	gui_drawpopup(x, y + 16, width, height, 16);
}

// gui_popupmenu: Draws a popup menu with text from a StringList
// ---------------------------------------------------------- >>
void gui_popupmenu(stringlist_t strings, char* title, ...)
{
	char string[256];
	va_list ap;

	va_start(ap, title);
	vsprintf(string, title, ap);
	va_end(ap);

	size_t max_len = strlen(string);

	if (max_len < strings.max_len)
		max_len = strings.max_len;

	int width = 32 + (8 * (int)max_len);
	int height = 32 + (10 * strings.n_strings);
	int x = (vid_width / 2) - (width / 2);
	int y = (vid_height / 2) - (height / 2);

	menu_area.set(x + 16, y + 32, x + width - 16, y + height);

	gui_drawpopup(x, y, width, height, string);
	draw_rect(rect_t(x + 16, y + 32 + (sel_item * 10), x + width - 16, y + 31 + ((sel_item + 1) * 10)), col_menuselect, true);

	for (DWORD s = 0; s < strings.n_strings; s++)
		draw_text(x + 16, y + 32 + (s * 10), col_menutext, 0, strings.strings[s]->c_str());
}

// gui_popupmessage: Draws a popup message with text from a StringList (1 string per line)
// ------------------------------------------------------------------------------------ >>
void gui_popupmessage(stringlist_t strings, const char* title, ...)
{
	char string[256];
	va_list ap;

	va_start(ap, title);
	vsprintf(string, title, ap);
	va_end(ap);

	size_t max_len = strlen(string);

	if (max_len < strings.max_len)
		max_len = strings.max_len;

	int width = 32 + (8 * (int)max_len);
	int height = 32 + (10 * strings.n_strings);
	int x = (vid_width / 2) - (width / 2);
	int y = (vid_height / 2) - (height / 2);

	gui_drawpopup(x, y, width, height, string);

	for (DWORD s = 0; s < strings.n_strings; s++)
		draw_text(x + 16, y + 32 + (s * 10), col_menutext, 0, strings.strings[s]->c_str());
}

// gui_drawmenu: Draws a menu with text from a StringList
// --------------------------------------------------- >>
void gui_drawmenu(stringlist_t strings, int x, int y)
{
	int width = 16 + (8 * (int)strings.max_len);
	int height = 16 + (10 * strings.n_strings);

	menu_area.set(x + 8, y + 8, x + width - 8, y + height - 8);

	gui_drawpopup(x, y, width, height, 8);

	//draw_box(x + 8, y + 7 + (sel_item * 10), x + width - 8, y + 7 + ((sel_item + 1) * 10), col_menuselect);
	draw_rect(rect_t(x + 8, y + 7 + (sel_item * 10), x + width - 8, y + 7 + ((sel_item + 1) * 10)), col_menuselect, true);

	for (DWORD s = 0; s < strings.n_strings; s++)
		draw_text(x + 8, y + 8 + (s * 10), col_menutext, 0, strings.strings[s]->c_str());
}

// gui_popupmenu_toggle: Draws a popup menu with text from a StringList and toggleable items
// -------------------------------------------------------------------------------------- >>
void gui_popupmenu_toggle(stringlist_t strings, char* title)
{
	size_t max_len = strlen(title);

	if (max_len < strings.max_len)
		max_len = strings.max_len;

	int width = 48 + (8 * (int)max_len);
	int height = 32 + (10 * strings.n_strings);
	int x = (vid_width / 2) - (width / 2);
	int y = (vid_height / 2) - (height / 2);

	menu_area.set(x + 16, y + 32, x + width - 16, y + height);

	gui_drawpopup(x, y, width, height, title);
	draw_rect(rect_t(x + 16, y + 31 + (sel_item * 10), width - 32, 10, 0), col_menuselect, true);

	for (DWORD s = 0; s < strings.n_strings; s++)
		draw_text(x + 32, y + 32 + (s * 10), col_menutext, 0, strings.strings[s]->c_str());

	for (DWORD s = 0; s < strings.n_strings; s++)
	{
		if (sel_items[s])
			draw_text(x + 16, y + 32 + (s * 10), col_menutext, 0, "*");
		else
			draw_text(x + 16, y + 32 + (s * 10), col_menutext, 0, "-");
	}
}

// gui_inputbox: Draws an input box
// ----------------------------- >>
void gui_inputbox(char *title, char *prompt)
{
	size_t max_len = strlen(title);

	if (max_len < strlen(prompt))
		max_len = strlen(prompt);

	int width = 32 + (8 * (int)max_len);
	int height = 64;
	int x = (vid_width / 2) - (width / 2);
	int y = (vid_height / 2) - (height / 2);

	gui_drawpopup(x, y, width, height, title);
	draw_text(x + 16, y + 40, col_menutext, 0, prompt);

	draw_rect(rect_t(x + 14, y + 50, width - 28, 14, 0), rgba_t(0, 0, 0, 80), true);
	draw_rect(rect_t(x + 16, y + 52, width - 32, 16, 0), COL_BLACK, true);
	draw_text(x + 16, y + 52, COL_WHITE, 0, menu_input.c_str());
}

// gui_menu: Draws the menu
// ---------------------- >>
void gui_menu()
{
	/*
	draw_texture_rect("_boxhead", 0, 0, 16, 16, rect_t(0, 0, 16, 16), col_menu);
	draw_texture_rect("_boxhead", 16, 0, vid_width - 32, 16, rect_t(16, 0, 48, 16), col_menu);
	draw_texture_rect("_boxhead", vid_width - 16, 0, 16, 16, rect_t(48, 0, 64, 16), col_menu);
	*/

	draw_texture_rect("_boxhead", 0, 0, vid_width, 16 * y_scale, rect_t(16, 0, 48, 16), col_menu);

	draw_text(9, 5, COL_BLACK, 0, "File Edit Options Mode Check");
	draw_text(8, 4, col_menuhead_text, 0, "File Edit Options Mode Check");
}

// gui_status_bar: Draws the status bar
// --------------------------------- >>
void gui_status_bar()
{
	glLineWidth(1.0f);

	draw_texture_rect("_boxhead", 0, vid_height - 12 * y_scale, vid_width, 12 * y_scale, rect_t(16, 0, 48, 16), col_menu);

	draw_text(vid_width - 128 * x_scale, vid_height - 10 * y_scale, col_menuhead_text, 0, "(%d, %d)", m_x(mouse.x), m_y(mouse.y));
	draw_line(rect_t(vid_width - 136 * x_scale, vid_height - 12 * y_scale, vid_width - 136 * x_scale, vid_height), rgba_t(0, 0, 0, 150), false);

	draw_text(vid_width - 224 * x_scale, vid_height - 10 * y_scale, col_menuhead_text, 0, "Grid: %d", gridsize);
	draw_line(rect_t(vid_width - 232 * x_scale, vid_height - 12 * y_scale, vid_width - 232 * x_scale, vid_height), rgba_t(0, 0, 0, 150), false);
}

// gui_vertexinfo: Draws the vertex information box
// --------------------------------------------- >>
void gui_vertexinfo(int v)
{
	int bottom = vid_height - 12;

	if (v == -1)
	{
		gui_drawbox(0, bottom - 144 * y_scale, 128 * x_scale, "");
		return;
	}

	gui_drawbox(0, bottom - 144 * y_scale, 128 * x_scale, "Vertex #%d", v);
	draw_text(16 * x_scale, bottom - 112 * y_scale, col_boxbody_text, 0, "X: %d", map.verts[v]->x);
	draw_text(16 * x_scale, bottom - 102 * y_scale, col_boxbody_text, 0, "Y: %d", map.verts[v]->y);
}

// gui_sectorinfo: Draws the sector information box
// --------------------------------------------- >>
void gui_sectorinfo(int s)
{
	int bottom = vid_height - 12;

	if (s == -1)
	{
		gui_drawbox(0, bottom - 144, 320, "");
		gui_drawbox(320, bottom - 144, 128, "");
		gui_drawbox(448, bottom - 144, 128, "");
		return;
	}

	gui_drawbox(0, bottom - 144, 320, "Sector #%d", s);
	draw_text(16, bottom - 112, col_boxbody_text, 0, "Floor: %d", map.sectors[s]->f_height);
	draw_text(16, bottom - 102, col_boxbody_text, 0, "Ceiling: %d", map.sectors[s]->c_height);
	draw_text(16, bottom - 92, col_boxbody_text, 0, "Height: %d", map.sectors[s]->c_height - map.sectors[s]->f_height);
	draw_text(16, bottom - 82, col_boxbody_text, 0, "Light: %d", map.sectors[s]->light);
	draw_text(16, bottom - 72, col_boxbody_text, 0, "Tag: %d", map.sectors[s]->tag);
	draw_text(16, bottom - 52, col_boxbody_text, 0, "Special: %d", map.sectors[s]->special);

	draw_text(16, bottom - 42, col_boxbody_text, 0, "%s", get_stype(map.sectors[s]->special).c_str());

	gui_drawbox(320, bottom - 144, 128, "Floor: %s", map.sectors[s]->f_tex.c_str());
	draw_rect(rect_t(334, bottom - 114, 434, bottom - 14), rgba_t(0, 0, 0, 40), true);
	draw_texture(map.sectors[s]->f_tex, 336, bottom - 112, 96, 96, COL_WHITE, true, true);

	gui_drawbox(448, bottom - 144, 128, "Ceil: %s", map.sectors[s]->c_tex.c_str());
	draw_rect(rect_t(462, bottom - 114, 562, bottom - 14), rgba_t(0, 0, 0, 40), true);
	draw_texture(map.sectors[s]->c_tex, 464, bottom - 112, 96, 96, COL_WHITE, true, true);
}

// thing_angle: Returns a string describing a numeric angle
// ----------------------------------------------------- >>
string thing_angle(short angle)
{

	if (angle == 0) // east
		return "East";
	else if (angle == 45) // northeast
		return "Northeast";
	else if (angle == 90) // north
		return "North";
	else if (angle == 135) // northwest
		return "Northwest";
	else if (angle == 180) // west
		return "West";
	else if (angle == 225) // southwest
		return "Southwest";
	else if (angle == 270) // south
		return "South";
	else if (angle == 315) // southeast
		return "Southeast";
	else // Invalid angle
		return "Invalid Angle";
}

// thing_difficulty: Adds a thing's difficulty levels to a string
// ----------------------------------------------------------- >>
void thing_difficulty(short flags, string *string)
{
	if (!(flags & THING_EASY)
		&& !(flags & THING_MEDIUM)
		&& !(flags & THING_HARD))
	{
		*string += "None";
		return;
	}

	if (flags & THING_EASY)
		*string += "Easy ";

	if (flags & THING_MEDIUM)
		*string += "Med ";

	if (flags & THING_HARD)
		*string += "Hard";
}

// thing_deaf: Returns a Yes/No string about a thing's deaf flag
// ---------------------------------------------------------- >>
string thing_deaf(short flags)
{
	if (flags & THING_DEAF)
		return "Yes";
	else
		return "No";
}

// thing_multi: Returns a Yes/No string about a thing's deaf flag
// -------------------------------------------------------- >>
string thing_multi(short flags)
{
	if (flags & THING_MULTI)
		return "Yes";
	else
		return "No";
}

// gui_thinginfo: Draws the thing information box
// ------------------------------------------- >>
void gui_thinginfo(int t)
{
	int bottom = vid_height - 12;
	string temp;
	char tempc[4] = "";

	if (t == -1)
	{
		gui_drawbox(0, bottom - 144, 384, "");
		return;
	}

	temp = get_tdef(map.things[t]->type)->name;
	gui_drawbox(0, bottom - 144, 384, "Thing #%d: %s", t, temp.c_str());
	draw_text(16, bottom - 112, col_boxbody_text, 0, "Type: %d", map.things[t]->type);
	draw_text(16, bottom - 102, col_boxbody_text, 0, "X: %d", map.things[t]->x);
	draw_text(16, bottom - 92, col_boxbody_text, 0, "Y: %d", map.things[t]->y);

	temp = "Angle: ";
	//temp += map.things[t]->angle;
	sprintf(tempc, "%d", map.things[t]->angle);
	temp += tempc;
	temp += " (";
	temp += thing_angle(map.things[t]->angle);
	temp += ")";
	draw_text(16, bottom - 82, col_boxbody_text, 0, temp.c_str());

	temp = "Difficulty: ";
	thing_difficulty(map.things[t]->flags, &temp);
	draw_text(16, bottom - 72, col_boxbody_text, 0, temp.c_str());

	temp = "Deaf: ";
	temp += thing_deaf(map.things[t]->flags);
	draw_text(16, bottom - 62, col_boxbody_text, 0, temp.c_str());

	draw_rect(rect_t(272, bottom - 112, 96, 96, RECT_TOPLEFT), rgba_t(0, 0, 0, 50), true);
	draw_texture_scale(get_tdef(map.things[t]->type)->sprite, 272, bottom - 112, 96, 96, COL_WHITE, false, false);

	if (!map.hexen)
	{
		temp = "Multiplayer: ";
		temp += bool2yesno(map.things[t]->flags & THING_MULTI);//thing_multi(map.things[t]->flags);
		draw_text(16, bottom - 52, col_boxbody_text, 0, temp.c_str());
	}
	else
	{
		temp = "Dormant: ";
		temp += bool2yesno(map.things[t]->flags & THING_DORMANT);
		draw_text(16, bottom - 52, col_boxbody_text, 0, temp.c_str());

		temp = "Game Modes: ";

		if (map.things[t]->flags & THING_SINGLE)
			temp += "SP ";

		if (map.things[t]->flags & THING_COOPERATIVE)
			temp += "CP ";

		if (map.things[t]->flags & THING_DEATHMATCH)
			temp += "DM ";

		draw_text(16, bottom - 42, col_boxbody_text, 0, temp.c_str());
		draw_text(16, bottom - 32, col_boxbody_text, 0, "Z Height: %d", map.things[t]->z);
		draw_text(16, bottom - 22, col_boxbody_text, 0, "Thing ID: %d", map.things[t]->tid);

		/*
		if (map.things[t]->type == 0)
			temp = "None";
		else
			temp = get_ltype(map.things[t]->special)->name;

		draw_text(16, bottom - 22, col_boxbody_text, 0, "Special: %s (%d)", temp, map.things[t]->special);
		*/

		/*
		temp = "Single Player: ";
		temp += bool2yesno(map.things[t]->flags & THING_SINGLE);
		draw_text(16, bottom - 42, col_boxbody_text, 0, temp.c_str());

		temp = "Cooperative: ";
		temp += bool2yesno(map.things[t]->flags & THING_COOPERATIVE);
		draw_text(16, bottom - 32, col_boxbody_text, 0, temp.c_str());

		temp = "Deathmatch: ";
		temp += bool2yesno(map.things[t]->flags & THING_DEATHMATCH);
		draw_text(16, bottom - 22, col_boxbody_text, 0, temp.c_str());
		*/
	}
}

// gui_checkmouse_line: Checks if the mouse is over 
bool gui_checkmouse_line(int l)
{
	int bottom = vid_height - 12;

	if (map.lines[l]->side1 != -1)
	{
		// First side upper
		if (mouse.x > 328 && mouse.x < 402 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 1;
			open_menu = MENU_SIDEMAIN;
			sel_item = 3;
			menu_action();
			return true;
		}

		// First side middle
		if (mouse.x > 403 && mouse.x < 477 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 1;
			open_menu = MENU_SIDEMAIN;
			sel_item = 2;
			menu_action();
			return true;
		}

		// First side lower
		if (mouse.x > 478 && mouse.x < 552 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 1;
			open_menu = MENU_SIDEMAIN;
			sel_item = 4;
			menu_action();
			return true;
		}
	}

	if (map.lines[l]->side2 != -1)
	{
		// Second side upper
		if (mouse.x > 568 && mouse.x < 642 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 2;
			open_menu = MENU_SIDEMAIN;
			sel_item = 3;
			menu_action();
			return true;
		}

		// Second side middle
		if (mouse.x > 643 && mouse.x < 717 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 2;
			open_menu = MENU_SIDEMAIN;
			sel_item = 2;
			menu_action();
			return true;
		}

		// Second side lower
		if (mouse.x > 718 && mouse.x < 626 &&
			mouse.y > bottom - 84 && mouse.y < bottom - 10)
		{
			line_side = 2;
			open_menu = MENU_SIDEMAIN;
			sel_item = 4;
			menu_action();
			return true;
		}
	}
	
	return false;
}

void gui_lineinfo(int l)
{
	int bottom = vid_height - 12;
	string temp;

	if (l == -1)
	{
		gui_drawbox(0, bottom - 144, 320, "");
		gui_drawbox(320, bottom - 144, 240, "");
		gui_drawbox(560, bottom - 144, 240, "");
		return;
	}

	// Line info
	if (!map.hexen)
	{
		gui_drawbox(0, bottom - 144, 320, "Line #%d", l);
		draw_text(16, bottom - 112, col_boxbody_text, 0, "Length: %d",
			(int)distance(map.l_getrect(l).x1(), map.l_getrect(l).y1(), map.l_getrect(l).x2(), map.l_getrect(l).y2()));
		draw_text(16, bottom - 102, col_boxbody_text, 0, "Front Sector: %d", map.l_getsector1(l));
		draw_text(16, bottom - 92, col_boxbody_text, 0, "Back Sector: %d", map.l_getsector2(l));
		draw_text(16, bottom - 82, col_boxbody_text, 0, "Sector Tag: %d", map.lines[l]->sector_tag);
		draw_text(16, bottom - 62, col_boxbody_text, 0, "Special %d:", map.lines[l]->type);

		if (map.lines[l]->type == 0)
			temp = "None";
		else
			temp = get_ltype(map.lines[l]->type)->name;

		draw_text(16, bottom - 52, col_boxbody_text, 0, "%s", temp.c_str());
	}
	else
	{
		gui_drawbox(0, bottom - 144, 320, "Line #%d", l);
		draw_text(16, bottom - 112, col_boxbody_text, 0, "Length: %d",
			(int)distance(map.l_getrect(l).x1(), map.l_getrect(l).y1(), map.l_getrect(l).x2(), map.l_getrect(l).y2()));
		draw_text(16, bottom - 102, col_boxbody_text, 0, "Front Sector: %d", map.l_getsector1(l));
		draw_text(16, bottom - 92, col_boxbody_text, 0, "Back Sector: %d", map.l_getsector2(l));

		if (map.lines[l]->type == 0)
			temp = "None";
		else
			temp = get_ltype(map.lines[l]->type)->name;

		draw_text(16, bottom - 82, col_boxbody_text, 0, "Special: (%d) %s", map.lines[l]->type, temp.c_str());

		for (int a = 0; a < 5; a++)
		{
			linetype_t *ltype = get_ltype(map.lines[l]->type);
			temp = ltype->args[a];

			if (ltype->arg_types[a] == NULL)
				draw_text(16, bottom - (72 - a*10), col_boxbody_text, 0, "%s: %d", temp.c_str(), map.lines[l]->args[a]);
			else
			{
				argtype_t *atype = get_argtype(*ltype->arg_types[a]);
				string arg = atype->get_name(map.lines[l]->args[a]);
				draw_text(16, bottom - (72 - a*10), col_boxbody_text, 0, "%s: %s", temp.c_str(), arg.c_str());
			}
		}

		draw_text(16, bottom - 22, col_boxbody_text, 0, "Trigger: %s", map.lines[l]->get_trigger_string().c_str());
	}

	// Front side
	gui_drawbox(320, bottom - 144, 240, "Front Side");
	draw_rect(rect_t(326, bottom - 96, 226, 88, RECT_TOPLEFT), rgba_t(0, 0, 0, 50), true);
	if (map.lines[l]->side1 != -1)
	{
		// Offsets
		draw_text(336, bottom - 112, col_boxbody_text, 0, "Offsets: %d, %d", map.l_getxoff(l, 1), map.l_getyoff(l, 1));

		// Upper tex
		draw_text(365, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side1]->tex_upper.c_str());
		if (map.l_needsuptex(l, 1))
			draw_texture_scale(map.sides[map.lines[l]->side1]->tex_upper, 328, bottom - 84, 74, 74, COL_WHITE, true, false);

		// Middle tex
		draw_text(440, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side1]->tex_middle.c_str());
		if (map.l_needsmidtex(l, 1))
			draw_texture_scale(map.sides[map.lines[l]->side1]->tex_middle, 403, bottom - 84, 74, 74, COL_WHITE, true, false);

		// Lower tex
		draw_text(515, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side1]->tex_lower.c_str());
		if (map.l_needslotex(l, 1))
			draw_texture_scale(map.sides[map.lines[l]->side1]->tex_lower, 478, bottom - 84, 74, 74, COL_WHITE, true, false);
	}

	// Back side
	gui_drawbox(560, bottom - 144, 240, "Back Side");
	draw_rect(rect_t(566, bottom - 96, 226, 88, RECT_TOPLEFT), rgba_t(0, 0, 0, 50), true);
	if (map.lines[l]->side2 != -1)
	{
		// Offsets
		draw_text(576, bottom - 112, col_boxbody_text, 0, "Offsets: %d, %d", map.l_getxoff(l, 2), map.l_getyoff(l, 2));

		// Upper tex
		draw_text(605, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side2]->tex_upper.c_str());
		if (map.l_needsuptex(l, 2))
			draw_texture_scale(map.sides[map.lines[l]->side2]->tex_upper, 568, bottom - 84, 74, 74, COL_WHITE, true, false);

		// Middle tex
		draw_text(680, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side2]->tex_middle.c_str());
		if (map.l_needsmidtex(l, 2))
			draw_texture_scale(map.sides[map.lines[l]->side2]->tex_middle, 643, bottom - 84, 74, 74, COL_WHITE, true, false);

		// Lower tex
		draw_text(755, bottom - 94, COL_BLACK, 1, map.sides[map.lines[l]->side2]->tex_lower.c_str());
		if (map.l_needslotex(l, 2))
			draw_texture_scale(map.sides[map.lines[l]->side2]->tex_lower, 718, bottom - 84, 74, 74, COL_WHITE, true, false);
	}
}
